-- by "Marhu" 
-- v 1.0.0
-- Date: 11.10.2013
-- WoolCollector platzierbar

WoolCollector = {};
WoolCollector.Dir = g_currentModDirectory;
WoolCollector_mt = Class(WoolCollector, Placeable);
InitObjectClass(WoolCollector, "WoolCollector");

function WoolCollector:new(isServer, isClient, customMt)
    local self = Placeable:new(isServer, isClient, WoolCollector_mt);
    registerObjectClassName(self, "WoolCollector");
	
	self.nodeID = 0;
	self.RollenMinY = -0.03;
	self.RollenMaxY = 0;
	self.transTime = 10000;
	
	return self;
end;

function WoolCollector:delete()
   
	if self.trigger then
		removeTrigger(self.trigger);
		self.trigger = nil;
	end;
	
	unregisterObjectClassName(self);
    WoolCollector:superClass().delete(self);
	
end;

function WoolCollector:deleteFinal()
    WoolCollector:superClass().deleteFinal(self);
end;

function WoolCollector:load(xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom)
    	
	if not WoolCollector:superClass().load(self, xmlFilename, x,y,z, rx,ry,rz, moveMode, initRandom) then
        return false;
    end;
	
	self.name = "WoolCollector"
		
	if not moveMode then
		
		local sx,sy,sz = getWorldTranslation(g_currentMission.husbandries.sheep.palletSpawnerPlaceId);
		local ex,ey,ez = getWorldTranslation(getChildAt(self.nodeId,0));
		local nx,ny,nz = getWorldTranslation(self.nodeId);		
		local distance1 = Utils.vector3Length(sx-ex, sy-ey, sz-ez);
		local distance2 = Utils.vector3Length(sx-nx, sy-ny, sz-nz);
		if distance1 > 0.35 or distance1 < 0.2 or distance2 < 4.4 then
			return false
		end
		
		self.TriggerCallback = WoolCollector.TriggerCallback;
		
		local xmlFile = loadXMLFile("TempXML", xmlFilename);
		
		local MainPlateIndex = getXMLString(xmlFile, "placeable.MainPlate#index");
		if MainPlateIndex ~= nil then         
			local MainPlate = Utils.indexToObject(self.nodeId, MainPlateIndex);
			if MainPlate ~= nil then
				self.Points = {};
				local numChildren = getNumOfChildren(MainPlate);
				for i = 1, numChildren do
					self.Points[i] = {};
					self.Points[i].node = getChildAt(getChildAt(MainPlate,i-1),0);
					self.Points[i].node2 = getChildAt(getChildAt(MainPlate,i-1),1);
					self.Points[i].set = false;
				end;
				self.Points[0] = {};
				self.Points[0].node2 = Utils.indexToObject(self.nodeId, getXMLString(xmlFile, "placeable.RollenSpawner#index"));
				self.Points[0].set = false;
				self.width = getUserAttribute(MainPlate, "width");
				local minY, maxY = Utils.getVectorFromString(getXMLString(xmlFile, "placeable.RollenMinMax#value"));
				if minY ~= nil and maxY ~= nil then
					self.RollenMinY = minY;
					self.RollenMaxY = maxY;
				end;
			else
				print("not MainPlate")
				return false;
			end;
		else
			print("not MainPlateIndex")
			return false;
		end;
		
		if self.isServer then	
			local TriggerIndex = getXMLString(xmlFile, "placeable.Trigger#index");
			if TriggerIndex ~= nil then         
				local Trigger = Utils.indexToObject(self.nodeId, TriggerIndex);
				if Trigger ~= nil then
					self.trigger = Trigger;
					addTrigger(self.trigger, "TriggerCallback", self);
				end;
			end;
			self.fremdObject = 0;
			self.PalletObjects = {};
		end;

		delete(xmlFile);
		
		if not WoolCollector.xmlFilenameLow then WoolCollector.xmlFilenameLow = {} end;
		table.insert(WoolCollector.xmlFilenameLow, string.lower(self.xmlFilename));
		self.xmlFilenameLow = string.lower(self.xmlFilename)
		self.WoolCollectorDirtyFlag = self:getNextDirtyFlag();
		setVisibility(g_currentMission.husbandries.sheep.palletSpawnerId,false)		
	end;
  
	return true;
end;

function WoolCollector:readStream(streamId, connection)
	
	WoolCollector:superClass().readStream(self, streamId, connection);
	
	for i = 0, table.getn(self.Points) do
		local set = streamReadBool(streamId);
		if self.Points[i].set ~= set then
			self.Points[i].set = set;
			local trans = self.RollenMinY;
			if set then trans = self.RollenMaxY end;
			setTranslation(self.Points[i].node2, 0,trans,0);
		end;
	end;
	
end;

function WoolCollector:writeStream(streamId, connection)
	
	WoolCollector:superClass().writeStream(self, streamId, connection);
	
	for i = 0, table.getn(self.Points) do
		streamWriteBool(streamId, self.Points[i].set);
	end;
	
end;

function WoolCollector:readUpdateStream(streamId, timestamp, connection)
	
	WoolCollector:superClass().readUpdateStream(self, streamId, timestamp, connection);
	
	if connection:getIsServer() then
		for i = 0, table.getn(self.Points) do
			local set = streamReadBool(streamId);
			if self.Points[i].set ~= set then
				self.Points[i].set = set;
				local trans = self.RollenMinY;
				if set then trans = self.RollenMaxY end;
				setTranslation(self.Points[i].node2, 0,trans,0);
			end;
		end;
	end;
	
end;

function WoolCollector:writeUpdateStream(streamId, connection, dirtyMask)
	
	WoolCollector:superClass().writeUpdateStream(self, streamId, connection, dirtyMask);
	
	if not connection:getIsServer() then
		for i = 0, table.getn(self.Points) do
			streamWriteBool(streamId, self.Points[i].set);
		end;
	end;
	
end;

function WoolCollector:update(dt)
	
	if self.isServer and self.Points[0].node then	
		
		if not self.firstScan then
			self.firstScan = true;
			for index,item in pairs(g_currentMission.itemsToSave) do
				if item.item:isa(FillablePallet) then
					local bx, by, bz = getWorldTranslation(item.item.nodeId);
					for i = 1, table.getn(self.Points) do
						local x, y, z = getWorldTranslation(self.Points[i].node);
						local distance = Utils.vector3Length(x-bx, y-by, z-bz);
						if distance < 1 then
							self.Points[i].Pallet = item.item.nodeId;
							if self.PalletScan and self.PalletScan[item.item.nodeId] then
								self.fremdObject = math.max(0,self.fremdObject - self.PalletScan[item.item.nodeId]);
							end;
							break;
						end;
					end;
				end;
			end;
			self.PalletScan = nil;
		end;
		
		if self.fremdObject <= 0 then	
			if self.Points[0].Pallet and not self.NextLine1 then
				for i = self.width - 1, 0, -1 do
					if self.Points[i].Pallet and (self.Points[i+1].Pallet == nil or self.Points[i+1].Move == true) then
						self.Points[i].Move = true;
						local sx,sy,sz = getWorldTranslation(self.Points[i].node);
						local ex,ey,ez = getWorldTranslation(self.Points[i+1].node);
						local trans = {getTranslation(self.Points[i].Pallet)};
						local newtrans = Utils.getMovedLimitedValues(trans, {ex,ey,ez}, {sx,sy,sz}, 3, self.transTime, dt, false);
						setTranslation(self.Points[i].Pallet, unpack(newtrans));
						local distance = Utils.vector3Length(ex-newtrans[1], ey-newtrans[2], ez-newtrans[3]);
						if distance <= 0.01 and self.Points[i+1].Pallet == nil then
							self.Points[i+1].Pallet = self.Points[i].Pallet;
							self.Points[i].Pallet = nil;
							self.Points[i].Move = nil;
						else
							self.Points[i].enable = true;
							self.Points[i+1].enable = true;
						end;
					end;
				end;
				
				local firstFull = true;
				for i = 1, self.width do
					if self.Points[i].Pallet == nil then
						firstFull = false;
						break;
					end;
				end;
				
				if firstFull == true then -- palett on 1 - 3 next line
					self.NextLine1 = true;
				end;
			elseif self.NextLine1 then
				self.NextLine1 = nil;
				local num = table.getn(self.Points) - self.width;
				for i = num, 1, -1 do
					if self.Points[i].Pallet and (self.Points[i+self.width].Pallet == nil or self.Points[i+self.width].Move == true)  then
						self.NextLine1 = true;
						self.Points[i].Move = true;
						local sx,sy,sz = getWorldTranslation(self.Points[i].node);
						local ex,ey,ez = getWorldTranslation(self.Points[i+self.width].node);
						local trans = {getTranslation(self.Points[i].Pallet)};
						local newtrans = Utils.getMovedLimitedValues(trans, {ex,ey,ez}, {sx,sy,sz}, 3, self.transTime, dt, false);
						setTranslation(self.Points[i].Pallet, unpack(newtrans));
						local distance = Utils.vector3Length(ex-newtrans[1], ey-newtrans[2], ez-newtrans[3]);
						if distance <= 0.01 and self.Points[i+self.width].Pallet == nil then
							self.Points[i+self.width].Pallet = self.Points[i].Pallet;
							self.Points[i].Pallet = nil;
							self.Points[i].Move = nil;
						else
							self.Points[i].enable = true;
							self.Points[i+self.width].enable = true;
						end;
					end;
				end;
				
				local firstEmpty = true;
				for i = 1, self.width do
					if self.Points[i].Pallet ~= nil then
						firstEmpty = false;
						break;
					end;
				end;
				if firstEmpty == true then
					self.NextLine1 = nil;
				end
			end;
		end;
			
		if g_currentMission.husbandries.sheep.currentPallet then
			local lvl = g_currentMission.husbandries.sheep.currentPallet.fillLevel;
			local capa = g_currentMission.husbandries.sheep.currentPallet.capacity;
			if lvl == capa then
				local AnotherHas = false;
				for i = 1, table.getn(WoolCollector.xmlFilenameLow) do
					if g_currentMission.placeables[WoolCollector.xmlFilenameLow[i]] then
						for k, v in pairs(g_currentMission.placeables[WoolCollector.xmlFilenameLow[i]]) do
							if v.Points[0].Pallet == g_currentMission.husbandries.sheep.currentPallet.nodeId then
								AnotherHas = true;
								break;
							end;
						end;
					end;
				end;
				if AnotherHas == false then
					for i = 1, table.getn(self.Points) do
						if self.Points[i].Pallet == nil then
							self.Points[0].Pallet = g_currentMission.husbandries.sheep.currentPallet.nodeId;
							break;
						end;
					end;
				end;
			end;
		end;
	elseif self.Points[0].node == nil then
		if g_currentMission.husbandries.sheep then
			self.Points[0].node = g_currentMission.husbandries.sheep.palletSpawnerPlaceId;
		end;
	end;
	
end;

function WoolCollector:updateTick(dt)
		
	if self.isServer and self.Points[0].node ~= nil then
		if g_currentMission.husbandries.sheep.currentPallet then
			local parent = getParent(g_currentMission.husbandries.sheep.currentPallet.nodeId);
			if getName(parent) ~= "RootNode" then
				local sheep = g_currentMission.husbandries.sheep;
				sheep:palletSpawnerTriggerCallback(sheep.palletSpawnerTriggerId, sheep.currentPallet.nodeId, false, true, nil);
				sheep.numObjectsInPalletSpawnerTrigger = 0;
			end;
		end;
		for i=0,table.getn(self.Points) do
			if self.Points[i].Pallet then
				local parent = getParent(self.Points[i].Pallet);
				if getName(parent) ~= "RootNode" then
					self.Points[i].Pallet = nil;
					self.Points[i].Move = nil;
				end;
			end;
			if self.Points[i].enable == true then
				self.Points[i].enable = false;
				if not self.Points[i].set then
					self.Points[i].set = true;
					setTranslation(self.Points[i].node2, 0,self.RollenMaxY,0);
					self.SendEvent = true;
				end;
			elseif self.Points[i].enable == false and self.Points[i].set == true then
				self.Points[i].set = false;
				setTranslation(self.Points[i].node2, 0,self.RollenMinY,0);
				self.SendEvent = true;
			end;
		end;
		if self.SendEvent then
			self.SendEvent = false;
			self:raiseDirtyFlags(self.WoolCollectorDirtyFlag);
		end;
	end;
	
end;

function WoolCollector:TriggerCallback(triggerId, otherId, onEnter, onLeave, onStay)
	
	if otherId ~= 0 then
		local object = g_currentMission:getNodeObject(otherId);
		if object ~= nil then
			if object:isa(FillablePallet) then
				if (onEnter) then 
					if otherId ~= self.Points[0].Pallet then
						if not self.firstScan then
							if not self.PalletScan then self.PalletScan = {} end;
							self.PalletScan[otherId] = (self.PalletScan[otherId] or 0) + 1;
						end;
						self.fremdObject = self.fremdObject + 1;
					else
						self.PalletObjects[otherId] = (self.PalletObjects[otherId] or 0) + 1;
					end;
				elseif (onLeave) then
					local find;
					for i =1 , table.getn(self.Points) do
						if self.Points[i].Pallet and self.Points[i].Pallet == otherId then
							find = true;
							self.Points[i].Pallet = nil;
							self.Points[i].Move = nil;
							if self.PalletObjects[otherId] then
								self.fremdObject = self.fremdObject + self.PalletObjects[otherId] - 1;
								self.PalletObjects[otherId] = nil;
							end;
						end;
					end;
					if find == nil then
						self.fremdObject = math.max(0,self.fremdObject - 1);
					end;
				end;
			else
				if (onEnter) then 
					self.fremdObject = self.fremdObject + 1;
				else
					self.fremdObject = math.max(0,self.fremdObject - 1);
				end;
			end;
		else
			if (onEnter) then 
				self.fremdObject = self.fremdObject + 1;
			else
				self.fremdObject = math.max(0,self.fremdObject - 1);
			end;
		end;
	end;
	
 end;

 
registerPlaceableType("WoolCollector", WoolCollector);